;;;; crunch - syntax for accessing C in crunched code

(module (crunch c)

  (c-include c-lambda c-external c-declare)

  (import (scheme base))
  (import-for-syntax (crunch compiler type)
  		     (matchable))
  (import (chicken foreign)
  	  (chicken syntax))

  (define-syntax c-include
    (er-macro-transformer
      (lambda (x r c)
        (let ((code (map (lambda (inc)
                           (string-append
                             "#include "
                             (if (symbol? inc)
                                 (symbol->string (strip-syntax inc))
                                 (string-append "\"" inc "\"\n"))))
                      (cdr x))))
          (if (memq #:crunched ##sys#features)
              `(##core#c-declare ,code)
              `(,(r 'foreign-declare) ,@code))))))

  (define-syntax c-lambda
    (er-macro-transformer
      (lambda (x r c)
        (match x
          ((_ (args ...) rtype body ...)
            (if (memq #:crunched ##sys#features)
                `(##core#c-lambda ,rtype ,args ,body)
                `(,(r 'foreign-lambda*)
                  ,(foreign-type (strip-syntax rtype))
                  ,(map (lambda (arg)
                          (list (foreign-type (car arg)) (cadr arg)))
                        args)
                  ,@body)))
          (_ (error 'c-lambda "invalid syntax" (strip-syntax x)))))))

  (define-syntax c-declare
    (er-macro-transformer
      (lambda (x r c)
        (if (memq #:crunched ##sys#features)
            `(##core#c-declare ,(cdr x))
            `(,(r 'foreign-declare) ,@(cdr x))))))

  (define-syntax c-external
    (er-macro-transformer
      (lambda (x r c)
        (define (conv name args type)
          `(,(r 'foreign-lambda)
             ,(foreign-type (strip-syntax type))
             ,name
             ,(map foreign-type (map strip-syntax args))))
        (match x
          ((_ include name (args ...) type)
            (if (memq #:crunched ##sys#features)
                `(##core#c-external ,@(cdr x))
                `(,(r 'begin)
                   (,(r 'c-include) ,(strip-syntax include))
                   ,(conv name args type))))
          ((_ name (args ...) type)
            (if (memq #:crunched ##sys#features)
                `(##core#c-external #f ,@(cdr x))
                (conv name args type)))))))

)
