;;;; crunch - utilities

(module (crunch compiler util) *

  (import (scheme base)
          (scheme write)
          (chicken base)
          (chicken format)
          (chicken condition))

  (define verbose (make-parameter #f))
  (define whole-program (make-parameter #f))
  (define error-handler (make-parameter abort))
  (define information-handler (make-parameter display))
  (define node-dump-hook (make-parameter void))

  (define (dribble fstr . args)
    (when (verbose)
      (flush-output-port)
      (elucidate (format "~?\n" fstr args))))

  (define (elucidate msg #!optional (port (current-error-port)))
    ((information-handler) msg port))

  (define crunch-condition? (condition-predicate 'crunch))

  (define (crunch-error msg filename args)
    ((error-handler)
      (make-property-condition
             'crunch
             'message msg
             'arguments args
             'filename filename)))

  (define get-error-message (condition-property-accessor 'crunch 'message))
  (define get-error-filename (condition-property-accessor 'crunch 'filename))
  (define get-error-arguments (condition-property-accessor 'crunch 'arguments))

  (define (report-crunch-error ex #!optional (port (current-error-port)))
    (fprintf port "~a: ~a\n"
      (get-error-filename ex)
      (get-error-message ex))
    (for-each
      (lambda (x) (fprintf port "  ~s\n" x))
      (get-error-arguments ex)))

)
