;;; test runner for CRUNCH

(import (scheme base)
        (crunch compiler)
        (chicken file)
        (chicken sort)
        (chicken pathname)
        (chicken format)
        (chicken process)
        (chicken condition)
        (chicken platform)
        (scheme process-context)
        (chicken file)
        compile-file)

(define failed #f)

(define utf.o "../crunch-utf.o")

(define c-options
  `("-I.." "-DCRUNCH_DEBUG" "-g" "-lm" "-fwrapv" "-Wno-unused-value"
    "-Wno-unused-command-line-argument" "-Wno-parentheses-equality" "-Wno-free-nonheap-object"))

(define csc-options
  `("-I.." "-C" "-DCRUNCH_DEBUG" "-C" "-I.."
    "-C" "-Wno-parentheses-equality" "-C" "-Wno-unused-value" "-C" "-Wno-free-nonheap-object"))

(for-each
  (lambda (fn)
    (printf "~a ...~!" fn)
    (call/cc
      (lambda (k)
        (handle-exceptions ex
          (begin
            (set! failed #t)
            (print-error-message ex)
            (k #f))
          (crunch fn
                  '(output-file "a.c")
                  `(compile-to-binary "a.out" ,utf.o ,@c-options))
          (unless (zero? (system "./a.out >out.txt"))
            (fprintf (current-error-port) "\nfailed\n")
            (set! failed #t))
          (let ((outf (pathname-replace-extension fn "out")))
            (when (file-exists? outf)
              (let ((out1 (with-input-from-file outf read-string))
                    (out2 (with-input-from-file "out.txt" read-string)))
                (unless (string=? out1 out2)
                  (fprintf (current-error-port) "\noutput differs:\n~a" out2)
                  (set! failed #t)))))
          (newline)))))
  (sort (glob "????-*.scm") string<?))

(printf "embedded ...~!")
(compile-file "embedded.scm" options: csc-options)
(newline)

(printf "import ...~!")
(crunch "import-external.scm"
        '(output-file "import-external.c")
        '(emit-import-library import-external)
        `(compile-to-binary "import-external.o" "-c" ,@c-options))
(crunch "import-test.scm"
        '(output-file "import-test.c")
        '(emit-import-library import-test)
        `(compile-to-binary "import-test.o" "-c" ,@c-options))
(crunch "use-import-test.scm"
        '(output-file "use-import-test.c")
        `(compile-to-binary "use-import-test"
                            "import-test.o" "import-external.o" ,utf.o ,@c-options))
(let ((input (with-input-from-pipe "./use-import-test" read-string)))
  (unless (string=? "246\n9.0\n" input)
    (fprintf (current-error-port) "\noutput differs from expected output:\n~a"
      input)
    (set! failed #t)))
(newline)

(when (member "quick" (command-line))
  (when failed (exit 1))
  (exit))

(printf "emulator ...~!")
(set! ##sys#include-pathnames (cons "6502" ##sys#include-pathnames))
(printf " building engine ...~!")
(load "6502/build.scm")
(printf " compiling ...\n")

(compile-file "6502/mos6502.scm" options: `("-J" ,@csc-options))
(compile-file "6502/forth.scm")

(for-each
  (lambda (fn)
    (when (file-exists? fn) (delete-file fn)))
  '("a.c" "a.out" "import-test.o" "import-test.c" "use-import-test"
    "use-import-test.c" "import-external.c" "import-external.o"
    "import-external.import.scm" "import-test.import.scm"))

(when failed (exit 1))
